/* http://www.jslint.com/lint.html */

/**
 * NOTE: This is a MOCK version of the full JSBridge, used for unit testing mpp_init.js
 */

/*jslint */
/*global privateMppBridge : false,
         console : true,
         prompt : true,
         setTimeout : true,
         document : true,
         mpp : true
*/

/**
 * Needed separately for jslint validation.
 */
var privateMppBridge,
    originalPrompt;

/**
 * @global
 * @namespace privateMppBridge
 */
privateMppBridge = privateMppBridge || (function () {
    "use strict";

        /**
         * Registered plugins
         *
         * @memberof privateMppBridge
         * @member {Object.<string,Plugin>} plugins
         * @private
         * @inner
         * @version 1
         */
    var plugins = {},

        /**
         * Returns JSON string of all registered plugins.
         *
         * @function listPlugins
         * @memberof privateMppBridge
         * @inner
         * @version 1
         *
         * @returns {String}
         */
        privateMppBridge_listPlugins = function () {
            var pluginList = [],
                pluginName,
                plugin;

            for (pluginName in plugins) {
                // http://jslinterrors.com/the-body-of-a-for-in-should-be-wrapped-in-an-if-statement
                if (plugins.hasOwnProperty(pluginName)) {
                    plugin = plugins[pluginName];
                    pluginList.push({
                        name : plugin.name,
                        version : plugin.version,
                        operations : plugin.operations
                    });
                }
            }

            /* Fixed response */
            return JSON.stringify({
                pluginInfos : pluginList
            });
        },

        /**
         * Returns JSON string of all registered plugins.
         *
         * @function invokePlugin
         * @memberof privateMppBridge
         * @inner
         * @version 1
         *
         * @param {String} callbackId - Unique string for use by callback
         * @param {String} operationJson - JSON string to invoke operation
         * @param {String} operationRequestJson - JSON string for request to operation
         * @returns {String} JSON string for invocation response
         */
        privateMppBridge_invokePlugin = function (callbackId, operation, operationRequestJson) {
            var operation,
                operationRequest,
                invokeResponse,
                executeResponse,
                operationResponse,
                plugin,
                pluginKey,
                startMillis = Date.now();

            try {
                operationRequest = JSON.parse(operationRequestJson);
                pluginKey = operation.pluginName + "@" + operation.pluginVersion;
                plugin = plugins[pluginKey];
                if (!plugin) {
                    throw new Error("Unknown plugin: [" + pluginKey + "]");
                }

                setTimeout(function () {
                    try {
                        operationResponse = plugin.invoke(operation.pluginOperationName, operationRequest);
                        executeResponse = {
                            error : false,
                            message : null,
                            durationMillis : Date.now() - startMillis
                        };
                    } catch (e) {
                        executeResponse = {
                            error : true,
                            message : JSON.stringify(e),
                            durationMillis : Date.now() - startMillis
                        };
                    }
                    mpp.privateFirePluginCallback(callbackId, JSON.stringify(executeResponse), JSON.stringify(operationResponse));
                }, 0);

                invokeResponse = {
                    error : false,
                    message : null
                };
            } catch (e) {
                invokeResponse = {
                    error : true,
                    message : JSON.stringify(e)
                };
            }

            return JSON.stringify(invokeResponse);
        },

        /**
         * @function callbackResponse
         * @memberof privateMppBridge
         * @inner
         * @version 1
         *
         * @param {String} callbackId - Unique string for use by callback
         * @param {boolean} isSuccess - Indicates whether callback was successful.
         * @param {String} message - Error message when not successful.
         * @returns {String} JSON string for invocation response
         */
        privateMppBridge_callbackResponse = function (callbackId, isSuccess, message) {
            if (!isSuccess) {
                console.warn("Failed in callback: callbackId: [" + callbackId + "], message: [" + message + "]");
            }
        },

        /**
         * @function isDebuggable
         * @memberof privateMppBridge
         * @inner
         * @version 1
         *
         * @returns {boolean} whether application can be debugged
         */
        privateMppBridge_isDebuggable = function () {
            return true;
        },

        Plugin = function (name, version, operationResponses) {
            var operations,
                operationName;

            this.name = name;
            this.version = version;
            this.operationResponses = operationResponses;

            operations = [];
            for (operationName in operationResponses) {
                // http://jslinterrors.com/the-body-of-a-for-in-should-be-wrapped-in-an-if-statement
                if (operationResponses.hasOwnProperty(operationName)) {
                    operations.push(operationName);
                }
            }

            this.operations = operations;
        },

        /**
         * @memberof privateMppBridge
         * @function init
         * @private
         * @inner
         * @version 1
         *
         * @returns {Object} exposed fields
         */
        privateMppBridge_init = function () {
            var exposedObject = {
                listPlugins : privateMppBridge_listPlugins,
                invokePlugin : privateMppBridge_invokePlugin,
                callbackResponse : privateMppBridge_callbackResponse,
                isDebuggable : privateMppBridge_isDebuggable
            };

            plugins["SleepPlugin@1"] = new Plugin("SleepPlugin", 1, {
                SLEEP : {
                    response : {
                        duration : 1,
                        unit : "SECONDS"
                    }
                }
            });
            plugins["CapabilitiesPlugin@1"] = new Plugin("CapabilitiesPlugin", 1, {
                GET_PAYMENT_PROVIDER_ELIGIBILITY : {
                    response : {
                        providers : {
                        }
                    }
                }
            });

            return exposedObject;
        };

    Plugin.prototype = {
        toString : function() {
            var result = [];

            result.push("name: [" + this.name + "]");
            result.push("version: [" + this.version + "]");
            result.push("operations: [" + this.operations + "]");

            return result.join(", ");
        },

        invoke : function(operationName, request) {
            var responses = this.operationResponses[operationName];

            if (!request) {
                console.log("Request was null for operation: [" + operationName + "]");
            }
            if (!responses) {
                throw new Error("No operation response found: [" + operationName + "] for plugin: [" + this.toString() + "]");
            }
            return responses.response;
        }
    };


    return privateMppBridge_init();
}());

originalPrompt = prompt;

/**
 * Simulator prompt function that replaces original JS prompt for testing purposes. This function simulates the functionality 
 * of the intercepted prompt in WebChromeClient.onJsPrompt in the Braavos Android code.
 *
 * @param message Parameter string for onJsPrompt. Used to pass a message to the Java code.
 * @return String returned by onJsPrompt when JsPromptResult.confirm is called in Java code.
 */
prompt = function (message) {
        var listPluginsFormat = "MPP_listPlugins",
            invokePluginFormat = "MPP_invokePlugin_",
            callbackResponseFormat = "MPP_callbackResponse_",
            isDebuggableFormat = "MPP_isDebuggable",
            payload;

        if (typeof message !== "string") {
            throw new Error("message is not a string!");
        } else if (message.indexOf(listPluginsFormat) === 0) {
            return privateMppBridge.listPlugins();
        } else if (message.indexOf(invokePluginFormat) === 0) {
            payload = JSON.parse(message.substr(invokePluginFormat.length));
            if (typeof payload !== "object") {
                throw new Error("Payload must be an object!");
            }
            return privateMppBridge.invokePlugin(payload.callbackId, payload.operation, payload.operationRequestJson);
        } else if (message.indexOf(callbackResponseFormat) === 0) {
            payload = JSON.parse(message.substr(callbackResponseFormat.length));
            if (typeof payload !== "object") {
                throw new Error("Payload must be an object!");
            }
            return privateMppBridge.callbackResponse(payload.callbackId, payload.isSuccess, payload.exceptionJson);
        } else if (message.indexOf(isDebuggableFormat) === 0) {
            return privateMppBridge.isDebuggable();
        } else {
            originalPrompt(message);
        }
    };
